/*
 * libunsigned - Unsigned MSSTYLES patching/support library for Win32/Win64
 * Copyright (C) 2007 Rafael Rivera Jr., http://www.anti-tgtsoft.com
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include "libunsigned.h"

// summary:		Locate target to patch (x86)

// description:	This is a fuzzy disassembler meant to crudely disassemble our way
//				to the beginning of the Verify(unsigned short const *, bool)
//				method (for the Intel/AMD 32-bit platform)

//				1. We start by jumping into exported function #29 begins
//				(undocumented CheckThemeSignature() entrypoint).

//				2. We then disassemble our way to the first JMP address we
//				see.

//				3. We then disassemble our way down past the constructor
//				of the CThemeSignature class to the actual Verify() method
//				and JMP to the method entry point.

//				4. We finally disassemble down before the stack is resized
//				(sub esp, 88h) and return the address for patching purposes.

// parameters:	Nothing

// returns:		Target address (for patching) or NULL if disassembly fails

#pragma region 32-bit platform disassembler
LONGLONG X86fuzzyTargetFinder()
{
	// dive into CheckThemeSignature()
	HMODULE hLib = LoadLibrary( TEXT("uxtheme.dll") );
	char* uxBaseAddress = (char*)GetProcAddress( hLib, (LPCSTR)LOWORD(29) );

	SWORD ss  = 0x00;
	BYTE byte = 0x00;
	BOOL bReturn = false, bSubScan = false;

	for( int i = 0; i < SCAN_THRESHOLD; i++ )
	{
		byte = *uxBaseAddress;
		switch( byte )
		{
			case OPCODE_MOV:
			case OPCODE_SUB_SIGNED:
				uxBaseAddress += 2;
			break;

			case OPCODE_LEA:
				uxBaseAddress += 3;
			break;

			case OPCODE_CALL:
				if( !bSubScan )
				{
					// skip offset bytes
					uxBaseAddress += 5;
					bSubScan = true;
					break;
				}

			case OPCODE_JMP:
				memcpy( &ss, uxBaseAddress + 1, sizeof(SWORD) );
				uxBaseAddress += 5 + ss; // move to next instr. then add offset
			break;

			case OPCODE_SUB_UNSIGNED:
				if( bSubScan )
					bReturn = true;
				else
					uxBaseAddress += 3; // skip subtraction
			break;

			default:
				uxBaseAddress++;
			break;
		}
	
		if( bReturn )
			return (LONGLONG)uxBaseAddress;
	}

	return NULL;
}
#pragma endregion

// summary:		Locate target to patch (AMD64)

// description:	This is a fuzzy disassembler meant to crudely disassemble our way
//				to the beginning of the Verify(unsigned short const *, bool)
//				method (for the AMD 64-bit platform)

//				1. We start by jumping into exported function #29 begins
//				(undocumented CheckThemeSignature() entrypoint).

//				2. We then disassemble our way to the first JMP address we
//				see.

//				3. We finally disassemble down before the stack is resized
//				(sub rsp, 78h) and return the address for patching purposes.

// parameters:	Nothing

// returns:		Target address (for patching) or NULL if disassembly fails
#pragma region AMD64 platform disassembler
LONGLONG AMD64fuzzyTargetFinder()
{
	// dive into CheckThemeSignature()
	HMODULE hLib = LoadLibrary( TEXT("uxtheme.dll") );
	char* uxBaseAddress = (char*)GetProcAddress( hLib, (LPCSTR)LOWORD(29) );

	SDWORD sd	= 0x00;
	BYTE byte	= 0x00;
	BOOL bReturn = false;

	for( int i = 0; i < SCAN_THRESHOLD; i++ )
	{
		byte = *uxBaseAddress;
		if( byte == X64_EXTENSION )
		{
			uxBaseAddress++;
			byte = *uxBaseAddress;
		}

		switch( byte )
		{
			case OPCODE_JMP:
				memcpy( &sd, uxBaseAddress + 1, sizeof(SDWORD) );
				uxBaseAddress += 5 + sd; // move to next instr. then add offset
			break;

			case OPCODE_MOV:
				uxBaseAddress += 2;
			break;

			case OPCODE_SUB_SIGNED:
				bReturn = true;
			break;

			default:
				uxBaseAddress++;
			break;
		}

		if( bReturn )
			return (LONGLONG)uxBaseAddress;
	}

	return NULL;
}
#pragma endregion

// Morgan W. is my friend.